@@ -10,14 +10,15 @@
 #endif
 
 typedef struct {
-    avifPixelFormat yuv_format;
+    avifPixelFormat subsampling;
     int qmin;
     int qmax;
-    int qmin_alpha;
-    int qmax_alpha;
     int speed;
     avifCodecChoice codec;
     avifRange range;
+    avifBool alpha_premultiplied;
+    int tile_rows_log2;
+    int tile_cols_log2;
 } avifEncOptions;
 
 // Encoder type
@@ -109,11 +110,23 @@ normalize_quantize_value(int qvalue) {
     }
 }
 
+static int
+normalize_tiles_log2(int value) {
+    if (value < 0) {
+        return 0;
+    } else if (value > 6) {
+        return 6;
+    } else {
+        return value;
+    }
+}
+
 static PyObject *
 exc_type_for_avif_result(avifResult result) {
     switch (result) {
         case AVIF_RESULT_INVALID_FTYP:
         case AVIF_RESULT_INVALID_EXIF_PAYLOAD:
+        case AVIF_RESULT_INVALID_CODEC_SPECIFIC_OPTION:
             return PyExc_ValueError;
         case AVIF_RESULT_BMFF_PARSE_FAILED:
         case AVIF_RESULT_TRUNCATED_DATA:
@@ -154,6 +167,32 @@ _encoder_codec_available(PyObject *self, PyObject *args) {
     return PyBool_FromLong(is_available);
 }
 
+static void
+_add_codec_specific_options(avifEncoder *encoder, PyObject *opts) {
+    Py_ssize_t i, size;
+    PyObject *keyval, *py_key, *py_val;
+    char *key, *val;
+    if (!PyTuple_Check(opts)) {
+        return;
+    }
+    size = PyTuple_GET_SIZE(opts);
+
+    for (i = 0; i < size; i++) {
+        keyval = PyTuple_GetItem(opts, i);
+        if (!PyTuple_Check(keyval) || PyTuple_GET_SIZE(keyval) != 2) {
+            return;
+        }
+        py_key = PyTuple_GetItem(keyval, 0);
+        py_val = PyTuple_GetItem(keyval, 1);
+        if (!PyBytes_Check(py_key) || !PyBytes_Check(py_val)) {
+            return;
+        }
+        key = PyBytes_AsString(py_key);
+        val = PyBytes_AsString(py_val);
+        avifEncoderSetCodecSpecificOption(encoder, key, val);
+    }
+}
+
 // Encoder functions
 PyObject *
 AvifEncoderNew(PyObject *self_, PyObject *args) {
@@ -162,55 +201,58 @@ AvifEncoderNew(PyObject *self_, PyObject *args) {
     AvifEncoderObject *self = NULL;
     avifEncoder *encoder = NULL;
 
-    char *yuv_format = "420";
+    char *subsampling = "4:2:0";
     int qmin = AVIF_QUANTIZER_BEST_QUALITY;    // =0
     int qmax = 10;                             // "High Quality", but not lossless
-    int qmin_alpha = AVIF_QUANTIZER_LOSSLESS;  // =0
-    int qmax_alpha = AVIF_QUANTIZER_LOSSLESS;  // =0
     int speed = 8;
     PyObject *icc_bytes;
     PyObject *exif_bytes;
     PyObject *xmp_bytes;
+    PyObject *alpha_premultiplied = NULL;
+    int tile_rows_log2 = 0;
+    int tile_cols_log2 = 0;
 
     char *codec = "auto";
     char *range = "full";
 
+    PyObject *advanced;
+
     if (!PyArg_ParseTuple(
             args,
-            "IIsiiiiissSSS",
+            "IIsiiissiiOSSSO",
             &width,
             &height,
-            &yuv_format,
+            &subsampling,
             &qmin,
             &qmax,
-            &qmin_alpha,
-            &qmax_alpha,
             &speed,
             &codec,
             &range,
+            &tile_rows_log2,
+            &tile_cols_log2,
+            &alpha_premultiplied,
             &icc_bytes,
             &exif_bytes,
-            &xmp_bytes)) {
+            &xmp_bytes,
+            &advanced)) {
         return NULL;
     }
 
-    if (strcmp(yuv_format, "4:0:0") == 0) {
-        enc_options.yuv_format = AVIF_PIXEL_FORMAT_YUV400;
-    } else if (strcmp(yuv_format, "4:2:0") == 0) {
-        enc_options.yuv_format = AVIF_PIXEL_FORMAT_YUV420;
-    } else if (strcmp(yuv_format, "4:2:2") == 0) {
-        enc_options.yuv_format = AVIF_PIXEL_FORMAT_YUV422;
-    } else if (strcmp(yuv_format, "4:4:4") == 0) {
-        enc_options.yuv_format = AVIF_PIXEL_FORMAT_YUV444;
+    if (strcmp(subsampling, "4:0:0") == 0) {
+        enc_options.subsampling = AVIF_PIXEL_FORMAT_YUV400;
+    } else if (strcmp(subsampling, "4:2:0") == 0) {
+        enc_options.subsampling = AVIF_PIXEL_FORMAT_YUV420;
+    } else if (strcmp(subsampling, "4:2:2") == 0) {
+        enc_options.subsampling = AVIF_PIXEL_FORMAT_YUV422;
+    } else if (strcmp(subsampling, "4:4:4") == 0) {
+        enc_options.subsampling = AVIF_PIXEL_FORMAT_YUV444;
     } else {
-        PyErr_Format(PyExc_ValueError, "Invalid yuv_format: %s", yuv_format);
+        PyErr_Format(PyExc_ValueError, "Invalid subsampling: %s", subsampling);
         return NULL;
     }
 
     enc_options.qmin = normalize_quantize_value(qmin);
     enc_options.qmax = normalize_quantize_value(qmax);
-    enc_options.qmin_alpha = normalize_quantize_value(qmin_alpha);
-    enc_options.qmax_alpha = normalize_quantize_value(qmax_alpha);
 
     if (speed < AVIF_SPEED_SLOWEST) {
         speed = AVIF_SPEED_SLOWEST;
@@ -251,6 +293,15 @@ AvifEncoderNew(PyObject *self_, PyObject *args) {
         return NULL;
     }
 
+    enc_options.tile_rows_log2 = normalize_tiles_log2(tile_rows_log2);
+    enc_options.tile_cols_log2 = normalize_tiles_log2(tile_cols_log2);
+
+    if (alpha_premultiplied == Py_True) {
+        enc_options.alpha_premultiplied = AVIF_TRUE;
+    } else {
+        enc_options.alpha_premultiplied = AVIF_FALSE;
+    }
+
     // Create a new animation encoder and picture frame
     self = PyObject_New(AvifEncoderObject, &AvifEncoder_Type);
     if (self) {
@@ -267,23 +318,31 @@ AvifEncoderNew(PyObject *self_, PyObject *args) {
         encoder->maxThreads = max_threads;
         encoder->minQuantizer = enc_options.qmin;
         encoder->maxQuantizer = enc_options.qmax;
-        encoder->minQuantizerAlpha = enc_options.qmin_alpha;
-        encoder->maxQuantizerAlpha = enc_options.qmax_alpha;
         encoder->codecChoice = enc_options.codec;
         encoder->speed = enc_options.speed;
         encoder->timescale = (uint64_t)1000;
+        encoder->tileRowsLog2 = enc_options.tile_rows_log2;
+        encoder->tileColsLog2 = enc_options.tile_cols_log2;
+
+#if AVIF_VERSION >= 80200
+        _add_codec_specific_options(encoder, advanced);
+#endif
+
         self->encoder = encoder;
 
         avifImage *image = avifImageCreateEmpty();
         // Set these in advance so any upcoming RGB -> YUV use the proper coefficients
         image->yuvRange = enc_options.range;
-        image->yuvFormat = enc_options.yuv_format;
+        image->yuvFormat = enc_options.subsampling;
         image->colorPrimaries = AVIF_COLOR_PRIMARIES_UNSPECIFIED;
         image->transferCharacteristics = AVIF_TRANSFER_CHARACTERISTICS_UNSPECIFIED;
         image->matrixCoefficients = AVIF_MATRIX_COEFFICIENTS_BT601;
         image->width = width;
         image->height = height;
         image->depth = 8;
+#if AVIF_VERSION >= 90000
+        image->alphaPremultiplied = enc_options.alpha_premultiplied;
+#endif
 
         if (PyBytes_GET_SIZE(icc_bytes)) {
             self->icc_bytes = icc_bytes;
@@ -395,6 +454,9 @@ _encoder_add(AvifEncoderObject *self, PyObject *args) {
         frame->yuvRange = image->yuvRange;
         frame->yuvFormat = image->yuvFormat;
         frame->depth = image->depth;
+#if AVIF_VERSION >= 90000
+        frame->alphaPremultiplied = image->alphaPremultiplied;
+#endif
     }
 
     frame->width = width;
@@ -818,6 +880,18 @@ setup_module(PyObject *m) {
     }
     Py_DECREF(v);
 
+    v = Py_BuildValue(
+        "(iii)",
+        AVIF_VERSION_MAJOR,
+        AVIF_VERSION_MINOR,
+        AVIF_VERSION_PATCH);
+
+    if (PyDict_SetItemString(d, "VERSION", v) < 0) {
+        Py_DECREF(v);
+        return -1;
+    }
+    Py_DECREF(v);
+
     if (PyType_Ready(&AvifDecoder_Type) < 0 || PyType_Ready(&AvifEncoder_Type) < 0) {
         return -1;
     }
